clear; clc; close all;
addpath(strcat(pwd,'/codes'))
addpath(strcat(pwd,'/annotation_info'))

Result_dir = pwd;

rand('seed', 10);
randn('seed', 10);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% generate simulation data from true isoforms from hg19.bed %%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% set model parameters
tau = 1.78;%% Poisson-Lognormal model parameter (precision)
alpha = 2; alpha0 = 0.5; v = 0.1;%% Gamma-Gamma model parameters (scale and shape)

%% set other experimental design factors
J1=50;J2=50;J=J1+J2;%% Number of samples in each condition
Num_genes = 300;%% Number of genes to be simulated
K = 2;%% Number of isoforms per gene
DEGPCENT=0.5;%% percentage of isoforms that are differenitally expressed

HAS_COV=1; %HAS_COV=1 means covraites is available, 0 means covariates is not avaiable.
th_C = 0.01; %percentage of contribution of covariates to the expression.
if HAS_COV==1
    age = randi([20,90], J, 1); %% demo ages range (20 to 90)
    sex = randi([0,1], J, 1); %% demo sex matrix
    treatment = [randi([0,1], J, 1),randi([0,1], J, 1)]; %% demo treatment, two types of treatment
    C = [age,sex,treatment]; %%covariate matrix
end

isoform_structure_file = 'isof_structure_forDemo.mat';
load(isoform_structure_file)

numGene = length(geneName);
I = cellfun(@(x) size(x,1), isofStruct,'UniformOutput',false);
I = cell2mat(I);
isofLen = cellfun(@sum, isofStruct, 'UniformOutput',false);
isofLen = cell2mat(isofLen);
READLEN = 100;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% simulate isoform abundance %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% randomly select genes to be simulated
id = find(I>10); %% genes with more than 10 exons
id = id(randperm(length(id), Num_genes*2));
G = length(id);
I = I(id);
X0 = isofStruct(id);
geneName1 = geneName(id);
isofName1 = isofName(id);
isofLen1 = isofLen(id,:);

Y0=cell(G,1);%% exon level counts in cell format
U=cell(G,1);%% within-sample dispersion parameter in cell format
BETA=zeros(K,G*J);%% isoform abundance beta
LAMBDA=zeros(G*K,2);%% scale parameter of beta

%% ground truth differential statues
d=double(rand(G*K,1)>(1-DEGPCENT));

%% simulate LAMBDA for the gamma-gamma model
for g=1:G
    for k=1:K
        if d((g-1)*K+k)==0
            lambda=gamrnd(alpha0,1/v,1,1);
            LAMBDA((g-1)*K+k,:)=lambda;
        else
            lambda1=gamrnd(alpha0,1/v,1,1);
            lambda2=gamrnd(alpha0,1/v,1,1);
            LAMBDA((g-1)*K+k,1)=lambda1;
            LAMBDA((g-1)*K+k,2)=lambda2;
        end
    end
end

%% simulate count model parameters beta
for g=1:G
    for k=1:K
        if d((g-1)*K+k)==0
            lambda=LAMBDA((g-1)*K+k,:);
            beta=gamrnd(alpha,1/lambda(1),1,J);
            if isofLen1(g,k)<READLEN
                beta=zeros(size(beta));
            end
            beta_tmp = beta;
            if HAS_COV == 1
                C_w = randn(1,size(C,2))/100; %%covariate weights
                C_w = C_w ./ mean(C,1);
                beta_C = C * C_w';
                beta_C = beta_C * min(abs(beta_tmp))/max(abs(beta_C))*th_C;
                beta_tmp = beta_tmp + beta_C';
            end
            BETA(k,(g-1)*J+1:g*J)=beta_tmp;
            BETA1((g-1)*K+k,:)=beta_tmp;
        else
            lambda1=LAMBDA((g-1)*K+k,1);
            lambda2=LAMBDA((g-1)*K+k,2);
            beta1=gamrnd(alpha,1/lambda1,1,J1);
            beta2=gamrnd(alpha,1/lambda2,1,J2);
            if isofLen1(g,k)<READLEN
                beta1=zeros(size(beta1));
                beta2=zeros(size(beta2));
            end
            beta_tmp = [beta1 beta2];
            if HAS_COV == 1
                C_w = randn(1,size(C,2))/100; %%covariate weights
                C_w = C_w ./ mean(C,1);
                beta_C = C * C_w';
                beta_C = beta_C * min(abs(beta_tmp))/max(abs(beta_C))*th_C;
                beta_tmp = beta_tmp + beta_C';
            end
            BETA(k,(g-1)*J+1:g*J)=beta_tmp;
            BETA1((g-1)*K+k,1:J1)=beta_tmp(1:length(beta1));
            BETA1((g-1)*K+k,J1+1:J)=beta_tmp(length(beta1)+1:end);
        end
        
    end
end

%% simulate count data Y0 in cell format
for g=1:G
    beta=BETA(:,(g-1)*J+1:g*J);
    U{g,1}=sqrt(1/tau).*randn(size(X0{g,1}));
    U{g,1}(X0{g,1}==0)=0;
    for k=1:K
        idx_tmp=find(U{g,1}(:,k)~=0);
        U_tmp=U{g,1}(idx_tmp,k);
        U_tmp=U_tmp-mean(U_tmp);
        U{g,1}(idx_tmp,k)=U_tmp;
        expU=exp(U{g,1});
        theta=X0{g,1}(:,k).*expU(:,k)*beta(k,:);
        y0{g,1}(:,:,k)=poissrnd(theta,size(theta,1),size(theta,2));
    end
end
for g=1:G
    Y0{g}=sum(y0{g},3);
end
fprintf('Done simulating count data!\n');
BETA_v=reshape(BETA,K*G*J,1);
BETA=mat2cell(BETA,K,ones(G,1)*J);
BETA=BETA';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% delete genes with extreme BETA values (optional)
BETA1_median=median(BETA1,1);
id2=[];
for i_BETA1=1:J
    id2=[id2 find(BETA1(:,i_BETA1)>BETA1_median(i_BETA1)*100)'];
end
id2=unique(id2);

id2=unique(ceil(id2/K));
id2=setdiff(1:G, id2);
id2=id2(1:min(length(id2),Num_genes));

y0=y0(id2,:);
Y0=Y0(id2,:);
U = U(id2);
BETA = BETA(id2);

G=length(id2);
I=I(id2);
X0=X0(id2);
geneName1=geneName1(id2);
isofName1=isofName1(id2);
isofLen1=isofLen1(id2,:);

for i_id2=1:length(id2)
    BETA1_tmp(K*(i_id2-1)+1:K*i_id2,:)=BETA1(K*(id2(i_id2)-1)+1:K*id2(i_id2),:);
    LAMBDA_tmp(K*(i_id2-1)+1:K*i_id2,:)=LAMBDA(K*(id2(i_id2)-1)+1:K*id2(i_id2),:);
    d_tmp(K*(i_id2-1)+1:K*i_id2,:)=d(K*(id2(i_id2)-1)+1:K*id2(i_id2),:);
end
BETA1=BETA1_tmp;
LAMBDA=LAMBDA_tmp;
d=d_tmp;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% END of simulation data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Run BayesIso with or without covariates

if HAS_COV == 1
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% If the data have covariate matrix, run BayesIso with covariates
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    fprintf('Running BayesIso with covariates...\n')
    [beta_e1, d_e] = func_BayesIso(Y0, X0, I,G,K,J,J1,J2,Result_dir,C);
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% If the data does not have covariate matrix, run BayesIso without covariates
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    fprintf('Running BayesIso...\n')
    [beta_e1, d_e] = func_BayesIso(Y0, X0, I,G,K,J,J1,J2,Result_dir);
end

%% performance evaluation
[rTP, rFP,AUC,PR_RC_F] = func_Eval_Perf(d_e, d,0.75,'prob');
h = figure;plot(rFP,rTP,'r','LineWidth',3)
xlabel('1-specificity','FontSize',14);ylabel('sensitivity','FontSize',14);title('ROC curve','FontSize',14)
h = figure;hist(d_e,30);
xlabel('Estimated probability for differential','FontSize',14);ylabel('Number of Isoforms','FontSize',14);title('Histogram of estimated probability of differential','FontSize',14)
save('res.mat')